#include <zephyr/bluetooth/bluetooth.h>
#include <bluetooth/mesh/models.h>
#include <bluetooth/mesh/dk_prov.h>
#include <dk_buttons_and_leds.h>
#include <zephyr/logging/log.h>
#include <zephyr/kernel.h>
#include <nrfx_power.h>

#define SLEEP_TIME        K_SECONDS(30)   // Czas snu - zwiększony dla oszczędności energii
#define WAKE_TIME         K_SECONDS(20)    // Krótszy czas aktywności

LOG_MODULE_REGISTER(chat, CONFIG_LOG_DEFAULT_LEVEL);
static bool mesh_active = false;

static void configure_low_power(void)
{
    // Konfiguracja niskiego poboru mocy dla nRF52832
    NRF_POWER->DCDCEN = 1;  // Włączenie przetwornika DC/DC
    NRF_POWER->TASKS_LOWPWR = 1;  // Przełączenie w tryb niskiego poboru mocy
}

static void set_adv_parameters(void)
{
    struct bt_le_adv_param adv_param = {
        .options = BT_LE_ADV_OPT_CONNECTABLE,
        .interval_min = 160,
        .interval_max = 1600,
    };
    int err = bt_le_adv_start(&adv_param, NULL, 0, NULL, NULL);
    if (err) {
        printk("Advertising failed to start (err %d)\n", err);
    }
}

static void bt_ready(int err)
{
    if (err) {
        printk("Bluetooth init failed (err %d)\n", err);
        return;
    }

   configure_low_power();
    
    err = bt_mesh_init(bt_mesh_dk_prov_init(), model_handler_init());
    if (err) {
        printk("Mesh initialization failed (err %d)\n", err);
        return;
    }

    if (IS_ENABLED(CONFIG_SETTINGS)) {
        settings_load();
    }

    bt_mesh_prov_enable(BT_MESH_PROV_ADV | BT_MESH_PROV_GATT);
    
    mesh_active = true;
    printk("Mesh initialized\n");
}

static void enter_sleep(void)
{
    if (!mesh_active) {
        return;
    }

    bt_le_adv_stop();
    bt_mesh_suspend();
    
    mesh_active = false;
    printk("Entering sleep mode\n");

    // Wykorzystanie trybu System OFF dla maksymalnej oszczędności energii
    nrf_power_system_off(NRF_POWER);
}

static void exit_sleep(void)
{
    configure_low_power();
    
    int err = bt_mesh_resume();
    if (err) {
        printk("Failed to resume Mesh (err %d)\n", err);
        bt_mesh_reset();
        bt_ready(0);
    } else {
        mesh_active = true;
        set_adv_parameters();
        bt_mesh_prov_enable(BT_MESH_PROV_ADV | BT_MESH_PROV_GATT);
        printk("Mesh resumed\n");
    }
}



static void enter_sleep(void)
{
    if (!mesh_active) {
        return;
    }

    bt_le_adv_stop();
    bt_mesh_suspend();
    
    mesh_active = false;
    printk("Entering sleep mode\n");

    // Zamiast System OFF, używamy deep sleep
    k_sleep(SLEEP_TIME);
}

static void exit_sleep(void)
{
    configure_low_power();
    
    int err = bt_mesh_resume();
    if (err) {
        printk("Failed to resume Mesh (err %d)\n", err);
        bt_mesh_reset();
        bt_ready(0);
    } else {
        mesh_active = true;
        set_adv_parameters();
        bt_mesh_prov_enable(BT_MESH_PROV_ADV | BT_MESH_PROV_GATT);
        printk("Mesh resumed\n");
    }
}

void main(void)
{
    int err = bt_enable(bt_ready);
    if (err) {
        printk("Bluetooth init failed (err %d)\n", err);
        return;
    }

    while (1) {
        if (bt_mesh_is_provisioned()) {
            LOG_INF("Mesh cycle start\n");
            exit_sleep();  // Włączamy mesh
            k_sleep(WAKE_TIME);
            enter_sleep();  // Wyłączamy mesh
        } else {
            printk("Waiting for provisioning...\n");
            k_sleep(K_MSEC(5000));  // Czekamy 5 sekund przed ponownym sprawdzeniem
        }
    }
}
